/*****************************************************************************/
/* FILE NAME: main.c                           COPYRIGHT (c) Freescale 2012  */
/*                                                      All Rights Reserved  */
/* DESCRIPTION: Main C program                                               */
/*                                                                           */
/*****************************************************************************/	
/* REV      AUTHOR        DATE        DESCRIPTION OF CHANGE                  */
/* ---   -----------    ----------    ---------------------                  */
/* 1.0	  D.McMenamin    13/Sept/12   Initial Public Release                 */
/*****************************************************************************/

#include "../headers/project.h"
#include "../headers/MPC5777m.h"
#include "fec.h"
#include "ethernetframe.h"

extern void MC_MODE_INIT(void);
extern void xcptn_xmpl();
extern void SDADC_INIT(void);
extern void DMA_Init(void);
extern int miiRead(int, uint16_t*);
extern uint32_t SDADC_GainCal(void);

// Declare external FEC descriptors and buffers
extern BufferDescriptor txDesc[2];
extern BufferDescriptor rxDesc[2];

extern uint8_t mac[6];
extern uint8_t broadcast[6];

extern uint8_t txBuf[1536];

// Global frame - points into the base address of the big txbuf
MyEthernetFrame *frame = NULL;		//define as NULL pointer otherwise it has trouble initialising


unsigned long lookup_16bit(unsigned long *, unsigned long);
short int sw_median(short int);


uint32_t duty_PIT;
uint32_t rx_duty;
uint8_t phase_PIT;
uint32_t dn1_PIT;
uint32_t tx_data;


void init_PIT5(void){
 PIT_0.TIMER[5].LDVAL.R = dn1_PIT;		/*Load value 65k interrupt 1.2kHz*/
 SIUL2.MSCR[0].R = 0x12380000; 				/*PWM LED*/
 SIUL2.GPDO[0].B.PDO = 0x1;					/*initially OFF*/
 PIT_0.TIMER[5].TCTRL.R = 0x00000003;		/*Turn timer and interrupt on*/
 INTC_0.PSR[231].R = 0xF001;
}

void init_PIT4(void){
 PIT_0.TIMER[4].LDVAL.R = 0x7A1200;		/*Load value 8M interrupt 10Hz*/
 PIT_0.TIMER[4].TCTRL.R = 0x00000003;		/*Turn timer and interrupt on*/
 INTC_0.PSR[230].R = 0xF002;
 SIUL2.MSCR[1].R = 0x12380000; 				/*flash LED to show int occurrence*/
 SIUL2.GPDO[1].B.PDO = 0x1;					/*initially OFF*/
}
// Disable the watchdog
void disableWatchdog() 
{
    SWT_0.SR.R = 0xC520;          /* Clear Soft lock bit in CR by writing     */
    SWT_0.SR.R = 0xD928;          /*  0xC520 followed by 0xD928 to WSC field  */
    SWT_0.CR.B.WEN = 0x0;         /* Clear Watchdog Enable Bit -> 0xC000010A  */
}

void setupGlobalFrame() 
{
	int m;
	
	frame = (void *) txBuf;
	
	
	// Setup txDesc for the (2x6-byte address + 2byte length + 4byte data) 18-byte frame
	txDesc[0].length = 18;
	txDesc[1].length = 18;
	
	// Setup broadcast as destination
	for (m = 0; m < 6; m++) 
	{
		frame->destination[m] = broadcast[m];
	}

	// Setup source address
	for (m = 0; m < 6; m++) 
	{
		frame->source[m] = mac[m];
	}

	// Set length (static at 4 for the 32-bit data payload)
	frame->length = 4;
	
	frame->data = tx_data;
	

}

void rxEthernetFrame() 
{
	// Find the active recieve buffer
	BufferDescriptor *rxBufDesc = getActiveRxDesc();
	
	// Extract the frame
	MyEthernetFrame *rxFrame = (void *) rxBufDesc->bufferPointer;

	// Do something with it, if we want to - we check if it's from an FSL OUI
	if (rxFrame->source[0] == 0x0 && 
			rxFrame->source[1] == 0x4 && rxFrame->source[2] == 0x9F) 
	{
		rx_duty = rxFrame->data; 	
	}

	// Mark the Rx buffer as available
	markRxDescEmpty(rxBufDesc);

	// Flag descriptors available to allow reception
	FEC.RDAR.R = 0x0100000;

	// Clear interrupt	
	FEC.EIR.R = 0x02000000;
}

void txEthernetFrame() 
{
	uint16_t x;
	frame->data = tx_data;			//take latest ADC result
	
	sendBuffer();		//and send
	
}

/************************************ Main ***********************************/
int main(){

 int i, j = 0;
 STM_0.CR.R = 0x00000001;				//system timer on for random MAC gen
 MC_MODE_INIT();              /* Configure Clocks and Modules Via Mode Entry */
 xcptn_xmpl();				//enable interrupts
 DMA_Init ();							  /* Configure DMA channel 16 to transfer SDADC1 results to SDADC1_RESULTS array*/
 
 SIUL2.MSCR[13].R = 0x70080000;                          /* PA13 for MAC config see setMacUsingStm() in fec.c*/
 SDADC_INIT ();
 
 /* Configure Pins for Clock out */
   SIUL2.MSCR[15].R = 0x32800001;                          /* PA15 as Sysclk1 can be used as clock for PHY*/
   SIUL2.MSCR[88].R = 0x22800001;                           /* PF8 as Sysclk0 */
   SIUL2.MSCR[3].R = 0x70080000;				//input button
 	disableWatchdog();

	setMacUsingStm();		//use STM to generate pseudo-random MAC address
	
	initFec();				//Init module and interface
	
	// Setup our frame that we'll send repeatedly
	setupGlobalFrame();
 
  
   
   SIUL2.MSCR[4].R = 0x12380000;		//used as ADC trigger
   
   duty_PIT = 0;		//duty cycle values for PWM
   dn1_PIT = 0;
   phase_PIT = 0;
   rx_duty = 0;
   

   
   init_PIT5();			//PIT5 controls PWM LED interrupt
   init_PIT4();			//PIT4 int triggers transmit frequency @ 10Hz
  

 	// Wait for connection to be confirmed by the PHY
	while(!isConnected());
	

SIUL2.GPDO[4].B.PDO = 1;		//Injection trigger enable



/* Main While Loop */
 while(1)
 {
  for (i = 0; i < 0xFFFF; i++);
  PIT_0.MCR.R = 0;			/*turn on timers, wait until init complete before going anywhere near tx*/
  if  (SDADC_1.SFR.B.DFFF == 0) 
  {
  SIUL2.GPDO[4].B.PDO = 1;		//Injection trigger enable
  }
  else 
  {
  tx_data = SDADC_1.CDR.B.CDATA;		//take ADC value directly for transmission
  SIUL2.GPDO[4].B.PDO = 0;
  }
}
}