#include "MPC5646C.h"

/**
 * Setup the EMIOS
 */
void initEMIOS(void) 
{
	EMIOS_0.MCR.B.MDIS = 0;				/* enable clks to eMIOS*/ 
	EMIOS_0.MCR.B.GPRE= 7;   			/* Divide 8 MHz FXOSC by 7+1 = 8 for 1MHz eMIOS clk */
	EMIOS_0.MCR.B.GPREN = 1;			/* Enable eMIOS clock */
	EMIOS_0.MCR.B.GTBE = 1;				/* Enable global time base */
	EMIOS_0.MCR.B.FRZ = 1;				/* Enable stopping channels when in debug mode */
}

/**
 * Setup the EMIOS Channel 0 to output a PWM signal - connect PJ5[0] (PA0 on device) to PJ8[5]
 * (PE4 on device) to see brightness of LED0 change as POT is adjusted
 */
void initEMIOSch0(void)     			/* EMIOS CH 0: Output Pulse Width Modulation (OPWMB) */
{      
	EMIOS_0.UCDIS.B.CHDIS0 = 0;			/* enable ch0*/
	EMIOS_0.CH[0].CBDR.R = 1023;    	/* Trailing edge when channel's counter bus=1023 */
	EMIOS_0.CH[0].CCR.B.BSL = 0x0;		/* Use counter bus A (default, setup in initEMIOSch23) */
	EMIOS_0.CH[0].CCR.B.EDPOL = 1;		/* Polarity-leading edge sets output/trailing clears */
	EMIOS_0.CH[0].CCR.B.MODE = 0x60;	/* Output Pulse Width Modulation Buffered (flag on B1 match) */
	SIU.PCR[0].R = 0x0600;      		/* Initialize pad for eMIOS chan. 1 output -> channel PA0 */
}

/**
 * Setup the EMIOS Channel 23 that provides Counter Bus A
 */
void initEMIOSch23(void)      			/* EMIOS CH 23: Modulus Up Counter */
{
	EMIOS_0.UCDIS.B.CHDIS23 = 0;		/* enable ch23*/
	EMIOS_0.CH[23].CADR.R = 1023;   	/* Period will be 1023+1 = 1024 clocks */
	EMIOS_0.CH[23].CCR.B.MODE = 0x50;	/* Modulus Counter Buffered (MCB) */
	EMIOS_0.CH[23].CCR.B.BSL = 0x3;		/* Use internal counter */
	EMIOS_0.CH[23].CCR.B.UCPRE=0;	   	/* Set channel prescaler to divide by 1 */
	EMIOS_0.CH[23].CCR.B.UCPEN = 1; 	/* Enable prescaler; uses default divide by 1 */
}
